'use client';

import { useLeaderboard } from '@/lib/hooks/use-leaderboard';
import { cn } from '@/lib/utils';
import { useMemo, useState } from 'react';
import AvatarComponent from './Avatar';
import Select from './Select';
import { type ScoreEvent } from '@/lib/leaderboard';
import useSWRSubscription, { type SWRSubscription } from 'swr/subscription';
import { useGroups } from '@/lib/hooks/use-groups';

const SCORE_COLORS = ['text-yellow-400', 'text-gray-400', 'text-orange-400'];

export default function Leaderboard({ token }: { token: string }) {
  // const { data, isLoading } = useLeaderboard({ token });

  const subscription: SWRSubscription<string, ScoreEvent, Error> = (key, { next }) => {
    const socket = new WebSocket(key);

    socket.addEventListener('message', (event) => {
      next(null, JSON.parse(event.data));
    });

    socket.addEventListener('error', (event) => {
      console.error(event);
    });

    return () => socket.close();
  };

  const { data } = useSWRSubscription(
    `wss://${process.env.NEXT_PUBLIC_API_URL?.split('//')[1]}/rleaderboard`,
    subscription
  );

  const [filter, setFilter] = useState('');

  let options = [] as { value: string; title: string }[];

  const { data: groups } = useGroups({ token });

  if (groups) {
    options = groups
      .filter((group) => !group.chapter)
      .map((group) => ({ value: group.name, title: group.name }))
      .filter((group, index, self) => self.findIndex((g) => g.value === group.value) === index)
      .sort((a, b) => a.title.localeCompare(b.title));

    options.unshift({ value: '', title: 'Tous' });
  }

  const filteredData = useMemo(() => {
    if (filter) {
      return data?.groups.filter((group) => group.name === filter);
    }
    return data;
  }, [data, filter]);

  console.log(filteredData);

  return (
    <section className="flex h-full w-full flex-col space-y-4">
      <header className="sticky flex items-center justify-between">
        <div className="flex flex-col">
          <h1 className="text-xl font-semibold">Tableau des scores</h1>
          <p className="text-muted">Suivez la progression des élèves en direct</p>
        </div>
        {/* {(filteredData && (
          <Select
            className="w-32"
            options={options || []}
            value={filter}
            onChange={(event) => setFilter(event.target.value)}
          />
        )) || (
          <span
            className="inline-block h-12 w-32 animate-pulse rounded-lg bg-primary-600"
            style={{
              animationDuration: '1s'
            }}
          />
        )} */}
      </header>
      <main className="flex flex-col justify-between space-x-0 space-y-4 pb-4">
        Waiting for the leaderboard to be implemented (Xavier WSS)
        {/* <ul className="flex flex-col space-y-2">
          {(!isLoading &&
            filteredData((group, key) => (
              <li key={key} className="flex justify-between space-x-2">
                <div className="flex items-center space-x-4">
                  <span className={cn('font-semibold', SCORE_COLORS[score.rank - 1])}>
                    {score.rank}
                  </span>
                  <div className="flex items-center space-x-2">
                    <AvatarComponent name={score.pseudo} src={score.avatar} className="h-9 w-9" />
                    <div className="flex flex-col gap-x-2 sm:flex-row sm:items-center">
                      <span className="text-lg">{score.pseudo}</span>
                      <span className="text-sm text-muted">
                        {score.groups
                          ?.map((g) => g.name)
                          .sort((a, b) => a.localeCompare(b))
                          .join(', ')}
                      </span>
                    </div>
                  </div>
                </div>
                <div className="flex items-center space-x-4">
                  <div className="flex flex-col">
                    <span className="text-sm font-semibold">Puzzles</span>
                    <span className="text-lg text-muted">{score.completions}</span>
                  </div>
                  <div className="flex flex-col">
                    <span className="text-sm font-semibold">Score</span>
                    <span className="text-lg text-muted">{score.score}</span>
                  </div>
                </div>
              </li>
            ))) ||
            [...Array(20).keys()].map((i) => (
              <span
                key={i}
                className="inline-block h-12 animate-pulse rounded-lg bg-primary-600"
                style={{
                  animationDelay: `${i * 0.05}s`,
                  animationDuration: '1s'
                }}
              />
            ))}
        </ul> */}
      </main>
    </section>
  );
}
